#include <sys/mman.h>
#include <stdio.h>
#include <xenctrl.h>
#include <stdlib.h>

#include <err.h>
#include <errno.h>
#include <fcntl.h>
#include <getopt.h>
#include <stdarg.h>
#include <string.h>
#include <termios.h>
#include <unistd.h>
#include <sys/ioctl.h>

//KAWASHO
#include <xenstore.h>

#define PATH_SEP '/'
#define MAX_PATH_LEN 256

#define true 1
#define false 0

#define XS_PERM_BOTH XS_PERM_READ | XS_PERM_WRITE

static char* getPermissionString(int permission){
	switch(permission){
	case XS_PERM_NONE:
		return "XS_PERM_NONE";
	case XS_PERM_READ:
		return "XS_PERM_READ";
	case XS_PERM_WRITE:
		return "XS_PERM_WRITE";
	case XS_PERM_BOTH:
		return "XS_PERM_READ | XS_PERM_WRITE (both)";
	case XS_PERM_ENOENT_OK:
		return "XS_PERM_ENOENT_OK";
	case XS_PERM_OWNER:
		return "XS_PERM_OWNER";
	}
}

static void chmod(char* path, int permission, int recurse, int domu, struct xs_handle *xsh, xs_transaction_t xth){
	int nperms = 1;
	struct xs_permissions perms[nperms];
	perms[0].id = 0;
	perms[0].perms = permission;

	char target[100];
	sprintf(target, path, domu);
	printf("%s %s", xs_set_permissions(xsh, xth, target, perms, nperms) ? "[   \e[32mOK\e[m   ] " : "[ \e[31mFAILED\e[m ] ", target);
	printf(" <- %s\n", getPermissionString(permission));

	if (recurse) {
		char buf[MAX_PATH_LEN];

		/* apply same permissions to all child entries: */
		unsigned int xsval_n;
		char **xsval = xs_directory(xsh, xth, target, &xsval_n);

		if (xsval) {
			int i;
			for (i = 0; i < xsval_n; i++) {
				snprintf(buf, MAX_PATH_LEN, "%s/%s", target, xsval[i]);

				chmod(buf, permission, true, domu, xsh, xth);
			}

			free(xsval);
		}
	}
}

int main(int argc, char** argv)
{
	xc_interface *xc_handle;
	uint32_t domr, domu;

	printf("\n     \x1b[33m\x1b[1mD-MORE\x1b[0m へようこそ  \n\n");
	fflush(stdin);

	if(argc != 3){
		fprintf(stderr, "使用法: %s Domain-R Domain-U\n", argv[0]);
		exit(1);
	}

	domr = strtoul(argv[1], NULL, 10);
	domu = strtoul(argv[2], NULL, 10);

	xc_handle = xc_interface_open(0,0,0);
	if(xc_handle == NULL){
		perror("xc_interface_open");
		exit(1);
	}

	int result = xc_domain_set_target(xc_handle, domr, domu);

	printf("ドメインR (%d) にドメインU (%d) へのメモリアクセス権限を付与しています...\n", domr, domu);
	printf("%s xc_domain_set_target: domM = %d, domU = %d (%d)\n\n",result > -1 ? "[   \e[32mOK\e[m   ] " : "[ \e[31mFAILED\e[m ] ", domr,domu, result);

	//KAWASHO added
	struct xs_handle *xsh = xs_open(XS_OPEN_SOCKETONLY);
	if (xsh == NULL) printf("XenStoreを開くことができませんでした！(xsh is NULL)\n");

	xs_transaction_t xth = xs_transaction_start(xsh);
	if (xth == XBT_NULL)
		printf("XenStore Transactionを確立できませんでした！(xth is XBT_NULL)\n");

	printf("必要なパーミッションを付与しています...\n");
	chmod("/local/domain/0/backend/console/%d/0", XS_PERM_BOTH, true, domu, xsh, xth);
	chmod("/local/domain/0/backend/vkbd/%d/0", XS_PERM_BOTH, true, domu, xsh, xth);
	chmod("/local/domain/0/backend/vfb/%d/0", XS_PERM_BOTH, true, domu, xsh, xth);
	chmod("/local/domain/0/device-model/%d/state", XS_PERM_BOTH, true, domu, xsh, xth);
	chmod("/local/domain/%d/device/vkbd/0", XS_PERM_READ, true, domu, xsh, xth);
	chmod("/local/domain/%d/device/vfb/0", XS_PERM_READ, true, domu, xsh, xth);
	chmod("/local/domain/%d/vm", XS_PERM_READ, false, domu, xsh, xth);
	chmod("/local/domain/%d/console/vnc-port", XS_PERM_BOTH, false, domu, xsh, xth);
	chmod("/local/domain/%d/console/vnc-listen", XS_PERM_BOTH, false, domu, xsh, xth);
	chmod("/local/domain/%d/console", XS_PERM_BOTH, true, domu, xsh, xth);

	printf("パーミッションの設定が完了しました！\n");
	xs_transaction_end(xsh, xth, 0);

	char com[MAX_PATH_LEN];
	sprintf(com, "/p/workspace/domainR-4.3.2/getbuffer.sh %d", domu);
	system(com);

	printf("完了！\n");

	return 0;
}

