#include <openssl/evp.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/mman.h>
#include <xenctrl.h>

#define PAGE_SIZE 4096

/**
 * コンパイル
 * gcc b.c -l xenctrl -std=gnu99 -l crypt -l ssl
 *
 * 実行
 * ./a.out [domU ID] 0xffffffff810000000 0xffffffff815a6cd1
 */
int main(int argc, char** argv) {
	xc_interface *xc_handle;

	uint32_t dom;
	unsigned long long begin_virt;
	unsigned long long end_virt;

	EVP_MD_CTX mdctx;
	unsigned int md_len;
	unsigned char md_value[EVP_MAX_MD_SIZE];

	OpenSSL_add_all_digests();

	if (argc != 4) {
		fprintf(stderr, "usage %s domid begin_addr end_addr\n", argv[0]);
		exit(1);
	}

	dom = strtoul(argv[1], NULL, 10);
	begin_virt = strtoul(argv[2], NULL, 16);
	end_virt = strtoul(argv[3], NULL, 16);

	begin_virt &= ~(PAGE_SIZE - 1);
	end_virt &=  ~(PAGE_SIZE - 1);

	xc_handle = xc_interface_open(0,0,0);
	if (xc_handle == NULL) { perror("xc_interface_open"); exit(1); }

	EVP_MD_CTX_init(&mdctx);
	EVP_DigestInit_ex(&mdctx, EVP_get_digestbyname("SHA1"), NULL);

	for (unsigned long long virt = begin_virt; virt <= end_virt; virt += PAGE_SIZE) {
		unsigned long mfn;
		void* map;

		mfn = xc_translate_foreign_address(xc_handle, dom, 0, virt);
		printf("MFNは%dです。\n", mfn);
		if (mfn == 0) { perror("xc_translate_foreign_address"); exit(1); }

		map = xc_map_foreign_range(xc_handle, dom, PAGE_SIZE, PROT_READ, mfn);
		if (map == NULL) { perror("xc_map_foreign_range"); exit(1); }

		EVP_DigestUpdate(&mdctx, map, PAGE_SIZE);

		if (munmap(map, PAGE_SIZE) < 0) { perror("munmap"); exit(1); }
	}

	EVP_DigestFinal_ex(&mdctx, md_value, &md_len);
	if (xc_interface_close(xc_handle) < 0) { perror("xc_interface_close"); exit(1); }

	printf("total memory: %f Mbytes\n", (end_virt - begin_virt)/1024/1024.0);

	for (int i = 0; i < md_len; i++) {
		printf("%02x ", md_value[i]);
	}
	printf("\n");


	printf("\n");

	return 0;
}
